import datetime
import logging
import os
import subprocess
from .config import loadControl

log = logging.getLogger()


def release():
    """ Release a package
    """
    version = loadControl()['version']
    hash = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD']).strip()
    date = datetime.datetime.now().strftime('%d/%m/%Y, %H:%M:%S')
    branch = 'release/v{version}-{hash}'.format(version=version, hash=hash)
    message = '"Release v{version} ({hash}) on {date}"'.format(version=version, hash=hash, date=date)
    message_noci = message[:-1] + ' [skip ci]"'

    command = 'git checkout -b {branch} develop'.format(branch=branch)
    log.debug(command)
    os.system(command)

    command = 'git checkout master'
    log.debug(command)
    os.system(command)

    command = 'git merge --no-ff {branch} --no-commit'.format(branch=branch)
    log.debug(command)
    os.system(command)

    command = 'git commit -m {message}'.format(message=message)
    log.debug(command)
    os.system(command)

    command = 'git tag -a v{version}-{hash} -m {message}'.format(version=version, hash=hash, message=message)
    log.debug(command)
    os.system(command)

    command = 'git checkout develop'
    log.debug(command)
    os.system(command)

    command = 'git merge --no-ff {branch} --no-commit'.format(branch=branch)
    log.debug(command)
    os.system(command)

    command = 'git commit -m {message}'.format(message=message_noci)
    log.debug(command)
    os.system(command)

    command = 'git branch -d {branch}'.format(branch=branch)
    log.debug(command)
    os.system(command)

    command = 'git push --all'
    log.debug(command)
    os.system(command)

    command = 'git push --tags'
    log.debug(command)
    os.system(command)



import logging
import os
import re
import sys

log = logging.getLogger()

repo_base_url = 'https://gitlab.it.geovar.net/wapt/packages/'

ok = {
    'package': False,
    'name': False,
    'version': False,
    'architecture': False,
    'section': False,
    'maintainer': False,
    'description': False,
    'description_fr': False,
    'description_es': False,
    'description_pt': False,
    'depends': False,
    'conflicts': False,
    'maturity': False,
    'locale': False,
    'target_os': False,
    'min_os_version': False,
    'max_os_version': False,
    'min_wapt_version': False,
    'sources': False,
    'installed_size': False,
    'impacted_process': False,
    'audit_schedule': False,
    'editor': False,
    'licence': False,
    'keywords': False,
    'homepage': False,
    'changelog': False
}


def controlCheck():
    """ Validate WAPT/control configuration file
    """
    if not os.path.isdir('WAPT'):
        log.critical('WAPT folder not found')
        sys.exit(1)

    path = os.path.join('WAPT', 'control')
    if not os.path.isfile(path):
        log.critical('WAPT/control file not found')
        sys.exit(1)

    package_name = ''

    check = True

    with open(path, 'r') as control:
        for line in control:
            key = line.rstrip().split(':', 1)[0].strip()
            if line[-2:] == '\r\n':
                log.error('WAPT/control {} line contains CRLF'.format(key))
                check = False
            else:
                value = line.rstrip().split(':', 1)[1].strip()
                log.debug('{} => {}'.format(key, value))

                if key == 'package':
                    if value.startswith('geovar-'):
                        ok[key] = True
                        package_name = value
                    else:
                        log.warning('WAPT/control {}: value do not start with geovar-'.format(key))

                elif key == 'name':
                    if not value:
                        log.warning('WAPT/control {}: value is empty'.format(key))
                    else:
                        ok[key] = True

                elif key == 'version':
                    if value.endswith('-0'):
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: value do not ends with -0'.format(key))

                elif key == 'architecture':
                    if re.match('^(all|x86|x64)$', value):
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: invalid value {}, should be (all|x86|x64)'.format(key, value))

                elif key == 'section':
                    if value == 'base':
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: invalid value {}, should be base'.format(key, value))

                elif key == 'maintainer':
                    if value == 'Geovariances IT Service (sysadmin@geovariances.com)':
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: invalid value'.format(key))

                elif key.startswith('description'):
                    if not value:
                        log.warning('WAPT/control {}: empty value'.format(key))
                    else:
                        if value[-1:] == '.':
                            ok[key] = True
                        else:
                            log.warning('WAPT/control {}: do not end with a dot'.format(key))

                elif key == 'depends':
                    if package_name != 'geovar-wapt-tools' and 'geovar-wapt-tools' not in value:
                        log.warning('WAPT/control {}: should contains geovar-wapt-tools'.format(key))
                    else:
                        ok[key] = True

                elif key == 'conflicts':
                    ok[key] = True

                elif key == 'maturity':
                    if value != 'BETA':
                        log.warning('WAPT/control {}: should always be BETA'.format(key))
                    else:
                        ok[key] = True

                elif key == 'locale':
                    ok[key] = True

                elif key == 'target_os':
                    ok[key] = True

                elif key == 'min_os_version':
                    ok[key] = True

                elif key == 'max_os_version':
                    ok[key] = True

                elif key == 'min_wapt_version':
                    ok[key] = True

                elif key == 'sources':
                    if value.startswith(repo_base_url):
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: should starts with {}'.format(key, repo_base_url))

                elif key == 'installed_size':
                    if not value or value.isdigit():
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: should be an integer'.format(key))

                elif key == 'impacted_process':
                    ok[key] = True

                elif key == 'audit_schedule':
                    if value.isdigit():
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: should be an integer'.format(key))

                elif key == 'editor':
                    if not value:
                        log.warning('WAPT/control {}: value is empty'.format(key))
                    else:
                        ok[key] = True

                elif key == 'licence':
                    if not value:
                        log.warning('WAPT/control {}: value is empty'.format(key))
                    else:
                        ok[key] = True

                elif key == 'keywords':
                    if not value:
                        log.warning('WAPT/control {}: value is empty'.format(key))
                    else:
                        ok[key] = True

                elif key == 'homepage' or key == 'changelog':
                    if value.startswith('https://'):
                        ok[key] = True
                    else:
                        log.warning('WAPT/control {}: should be an url'.format(key))

                else:
                    log.warning('WAPT/control {}: unwanted key'.format(key))
                    check = False

    for key in ok.keys():
        if not ok[key]:
            log.error('WAPT/control {}: invalid or missing value'.format(key))
        check &= ok[key]

    log.debug('check = {}'.format(check))

    return check



import argparse
import logging
import os
import sys
from .vers import __version__
from .check import isPackage
from .version import versionChecker
from .validate import validate
from .create import create
from .build import build
from .release import release
from .hash import sha256sum

log = logging.getLogger()


def cli():
    parser = argparse.ArgumentParser(prog='wapt', description='WAPT packaging utility.')
    subparsers = parser.add_subparsers(
        title='commands',
        dest='command')

    subparsers.add_parser(
        'version',
        help='display cli version')

    parser_check = subparsers.add_parser(
        'check-version',
        help='check if new upstream package exists')

    parser_check.add_argument(
        '--chat',
        help='send results to chat',
        action='store_true')

    parser_check.add_argument(
        '--badge',
        help='generate badge',
        action='store_true')

    subparsers.add_parser(
        'validate',
        help='validate package')

    parser_new = subparsers.add_parser(
        'create',
        help='create a new package')

    parser_new.add_argument(
        '--name',
        help='name of the new package to create',
        type=str)

    parser_build = subparsers.add_parser(
        'build',
        help='build package from current folder')

    parser_build.add_argument(
        '--key',
        help='key path to sign package',
        type=str)

    parser_build.add_argument(
        '--release',
        help='release number of package',
        type=int)

    parser_build.add_argument(
        '--maturity',
        help='maturity of package, overrides WAPT/control content',
        type=str)

    parser_upload = subparsers.add_parser(
        'upload',
        help='upload package')

    parser_upload.add_argument(
        '--server',
        help='WAPT server url',
        type=str)

    parser_upload.add_argument(
        '--user',
        help='WAPT server url',
        type=str)

    subparsers.add_parser(
        'release',
        help='release package to production')

    subparsers.add_parser(
        'setup',
        help='initialize folder after git clone')

    parser_checksum = subparsers.add_parser(
        'checksum',
        help='compute checksum')

    parser_checksum.add_argument(
        '--file',
        help='file to be analyzed',
        type=str)

    parser.add_argument('--verbose', help='verbose output', action='store_true')
    parser.add_argument('--silent', help='no output except critical messages', action='store_true')

    args = parser.parse_args()

    logging_config = logging.StreamHandler(sys.stdout)
    logging_config.setFormatter(logging.Formatter('[%(asctime)s - %(levelname)8s] %(message)s'))
    log.addHandler(logging_config)

    if args.silent:
        log.setLevel(logging.CRITICAL)
    elif args.verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)

    log.info('WAPT Tools CLI v{}'.format(__version__))

    if args.command == 'version':
        sys.exit(0)

    if args.command == 'check-version':
        if not isPackage():
            sys.exit(1)

        mismatch = versionChecker(chat=args.chat, badge=args.badge)
        if mismatch and 'CI' in os.environ:
            sys.exit(1)
        else:
            sys.exit(0)

    if args.command == 'validate':
        if not isPackage():
            sys.exit(1)

        if not validate():
            log.error('not a valid package')
            sys.exit(1)
        else:
            log.info('valid package')
            sys.exit(0)

    if args.command == 'create':
        create(args.name)
        sys.exit(0)

    if args.command == 'build':
        build(key=args.key, release=args.release, maturity=args.maturity)
        sys.exit(0)

    if args.command == 'release':
        if not isPackage():
            sys.exit(1)

        release()
        sys.exit(0)

    if args.command == 'setup':
        command = 'git flow init --defaults --force'
        log.debug(command)
        os.system(command)
        sys.exit(0)

    if args.command == 'checksum':
        checksum = sha256sum(args.file)
        log.info('{file} checksum is {checksum}'.format(file=args.file, checksum=checksum))
        sys.exit(0)



#from .build import * # noqa
from .config import * # noqa
from .commands import * # noqa
from .download import * # noqa
from .version import * # noqa
from .hash import * # noqa
from .log import * # noqa



import anybadge
import bs4
import logging
import json
import re
import requests
import os
from .config import loadControl, loadVersionCheck

log = logging.getLogger()

chat_message = """
New version of *{package}* have beed released.

Current version: {old_version}
Latest version: *{new_version}*

Jump to {homepage}
"""


def latestVersion():
    """ Extract latest version defined by version-check.json

    Returns
    -------
    version: string
        version extracted from web page
    """
    config = loadVersionCheck()

    content = requests.get(config['url_version']).text.strip()

    if config['html'] is True:
        soup = bs4.BeautifulSoup(content, 'html.parser')
        if 'index' in config:
            index = config['index']
        else:
            index = 0

        latest_version = soup.select(config['selector'])[index].contents[0].strip()

        # log.debug('index = {}'.format(index))
        # selected = soup.select(config['selector'])
        # log.debug('selected = {}'.format(selected))
        # log.debug('selected[{}] = {}'.format(index, selected[index]))
        # log.debug('selected[{}].contents[0] = {}'.format(index, selected[index].contents[0]))
        # latest_version = selected[index].contents[0].strip()
    else:
        latest_version = content

    if 'cleaners' in config and len(config['cleaners']) > 0:
        for cleaner in config['cleaners']:
            latest_version = re.sub(cleaner['pattern'], cleaner['rewrite'], latest_version, flags=re.DOTALL).strip()

    return latest_version


def versionChecker(chat=False, badge=False):
    """ Compare latest version defined by version-check.json, versus WAPT/control one

    Parameters
    ----------
    chat: bool
        send results to chat
    badge: bool
        generate upstream badge

    Returns
    -------
    results: bool
        version mismatch
    """
    log.debug('versionChecker')

    control = loadControl()
    log.info('Current {} version: {}'.format(control['name'], control['version']))

    latest_version = latestVersion()
    log.info('Latest {} version: {}'.format(control['name'], latest_version))

    if control['version'] != latest_version:
        log.info('New version available, please upgrade package')

        if chat:
            log.debug('Sending message to Chat webhook ...')
            if 'CHAT_WEBHOOK_URL' in os.environ:
                requests.session().post(
                    os.environ['CHAT_WEBHOOK_URL'],
                    data=json.dumps({
                        'text': chat_message.format(
                            package=control['name'],
                            old_version=control['version'],
                            new_version=latest_version,
                            homepage=control['homepage'])
                    }),
                    headers={'Content-Type': 'application/json; charset=UTF-8'}
                )
            else:
                log.warning('CHAT_WEBHOOK_URL environment variable not defined, unable to send chat message')
            log.debug('... done.')

        if badge:
            log.debug('generating badge ...')
            badge = anybadge.Badge(label='upstream', value='new', default_color='red')
            badge.write_badge('upstream.svg')

        return True

    if badge:
        log.debug('generating badge ...')
        badge = anybadge.Badge(label='upstream', value='ok', default_color='green')
        badge.write_badge('upstream.svg')

    return False



import hashlib
import os
import logging
from .check import isPackage
from .control import controlCheck

log = logging.getLogger()


def sha256sum(filename):
    h = hashlib.sha256()
    b = bytearray(128*1024)
    mv = memoryview(b)
    with open(filename, 'rb', buffering=0) as f:
        for n in iter(lambda: f.readinto(mv), 0):
            h.update(mv[:n])
    return h.hexdigest()


def build(key, release, maturity):
    """ Build package

    Parameters
    ----------
    key: string
        pathname of signing private key
    release: int
        release number
    maturity: string
        maturity of the package
    """
    if not isPackage():
        log.error('current folder does not contain a WAPT package')
        return False

    if not controlCheck():
        log.error('WAPT/control invalid')
        return False

    log.debug('List files from current folder')
    list = []
    for root, dirs, files in os.walk('.'):
        for filename in files:
            if root == '.':
                if filename != '.git':
                    list.append(os.path.join(root, filename))
                    print('{}/{}'.format(root, filename))
            elif root == './WAPT':
                if filename != 'manifest.sha256':
                    list.append(os.path.join(root, filename))
                    print('{}/{}'.format(root, filename))
            elif root == './sources':
                if filename != '.gitignore':
                    list.append(os.path.join(root, filename))
                    print('{}/{}'.format(root, filename))
            elif root == './config':
                if filename != '.gitkeep':
                    list.append(os.path.join(root, filename))
                    print('{}/{}'.format(root, filename))

    log.debug('Creating WAPT/manifest.sha256')

    content = '['
    for file in list:
        hash256 = sha256sum(file)
        content += '["{}", "{}"],'.format(file[2:], hash256)
    content = content[:-1] + ']'

    with open(os.path.join('WAPT', 'manifest.sha256'), 'w') as output:
        output.write(content)

# printf "\n Creation of \"manifest.sha1\"\n"
# create_manifest_sha1 > WAPT/manifest.sha1
#   ## On enleve la virgule à l'avant derniere ligne.
# sed -i 'x; ${s/,//;p;x}; 1d' WAPT/manifest.sha1
# printf "\n File signature of manifest.sha1 with ${PRIVATE_KEY}\n"
# sign_manifest > WAPT/signature
# printf "\n Zipping folder as <wapt>.wapt\n"
# zip_to_wapt
# upload_package "${PACKAGE_package}_${PACKAGE_version}_${PACKAGE_architecture}"
# rm -f "${TEMP_FILE}"

    return True



import logging
import sys
from .version import versionChecker

log = logging.getLogger()


def commands(downloader=None):
    """ Default main function for WAPT setup.py
    """

    logging_config = logging.StreamHandler(sys.stdout)
    logging_config.setFormatter(logging.Formatter('[%(asctime)s - %(levelname)8s] %(message)s'))
    log.addHandler(logging_config)
    log.setLevel(logging.INFO)

    if len(sys.argv) == 1:
        if downloader is not None:
            downloader()
    else:
        if sys.argv[1] == 'version-check':
            versionChecker()



import logging
import os


def logger(package):
    logger = logging.getLogger(package)
    logger.setLevel(logging.DEBUG)
    logfile = os.path.join(os.environ['SYSTEMROOT'], 'WAPT', 'logs', package + '.log')
    os.makedirs(os.path.dirname(logfile))
    handler = logging.FileHandler(logfile)
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(funcName)s: %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger



import glob
import logging
import os
import requests
import sys
from .hash import sha256sum

log = logging.getLogger()


def save_url_to_file(url, pathname):
    try:
        r = requests.get(url, stream=True)
        if not r.ok:
            log.critical('failed to download {} from {}, 404 not found'.format(os.path.basename(pathname), url))
            sys.exit(1)

        with open(pathname, 'wb') as f:
            for chunk in r.iter_content(1024):
                f.write(chunk)
        del r
    except Exception as e:
        log.critical('failed to download {} from {}, {}'.format(os.path.basename(pathname), url, e))
        sys.exit(1)


def download(url, pathname, clean=True, checksum=None):
    """ Download the file at a given url and save it to pathname.

    The files existing in the same folder and with the same extension are removed if clean=True.

    Parameters
    ----------
    url: string
        url of the file to be dowloaded
    pathname: string
        pathname of the file to be saved
    clean: boolean
        cleanup needed
    """
    extension = os.path.splitext(pathname)[1]

    dirname = os.path.dirname(pathname)

    if clean:
        pattern = os.path.join(dirname, '*{}'.format(extension))
        log.debug('cleaning files {} except {}'.format(pattern, pathname))
        for file in glob.glob(pattern):
            if file != pathname:
                log.debug('removing {}'.format(file))
                os.remove(file)

    log.info('downloading {} from {}'.format(os.path.basename(pathname), url))

    if not os.path.exists(pathname):
        save_url_to_file(url, pathname)
    else:
        if checksum:
            computed = sha256sum(pathname)
            if computed != checksum:
                log.info('{file} checksum mismatch, downloading'.format(file=pathname))
                os.remove(pathname)
                save_url_to_file(url, pathname)
        else:
            log.debug('{} already exists'.format(pathname))

    if checksum:
        computed = sha256sum(pathname)
        if computed != checksum:
            os.remove(pathname)
            log.critical('{file} checksum mismatch, downloaded file deleted'.format(file=pathname))
            sys.exit(1)



import logging
import gitlab
import os
import random
import sys
from pkg_resources import resource_string

log = logging.getLogger()

pipeline = '[![{text} status]({url}/wapt/packages/{package}/badges/{branch}/pipeline.svg?key_text={text})]({url}/wapt/packages/{package}/-/pipelines?&ref={branch})\n' # noqa
upstream = '![{text}]({url}/wapt/packages/{package}/-/jobs/artifacts/develop/raw/upstream.svg?job=version-check)'


def create(package, verbose=False):
    """ Create a new package
    """
    gl = gitlab.Gitlab.from_config()
    log.debug('Gitlab url: {}'.format(gl.url))

    if not package:
        log.critical('Package name missing, use --name')
        sys.exit(1)

    # Check if gitlab project exists
    projects = gl.projects.list(search=package)
    if len(projects) > 0:
        for project in projects:
            if project.name == package:
                log.error('Gitlab project {} already exists, aborting.'.format(package))
                sys.exit(1)

    if os.path.isdir(package):
        log.error('folder {} already exists, aborting.'.format(package))
        sys.exit(1)

    wapt_group = gl.groups.list(search='wapt')[0]
    packages_group = wapt_group.subgroups.list(search='packages')[0]
    project = gl.projects.create({'name': package, 'namespace_id': packages_group.id})

    command = 'git clone {}:wapt/packages/{}'.format(gl.url.replace('https://', 'git@'), package)
    log.debug(command)
    os.system(command)
    os.chdir(package)

    log.debug('creating README.md file')
    with open('README.md', 'w') as file:
        file.write('# WAPT {} package\n'.format(package))
        file.write('\n')
        file.write(pipeline.format(text='PROD', branch='master', url=gl.url, package=package))
        file.write(pipeline.format(text='DEV', branch='develop', url=gl.url, package=package))
        file.write(upstream.format(text='Upstream', url=gl.url, package=package))

    command = 'git add -A'
    log.debug(command)
    os.system(command)

    command = 'git commit -m "Add README.md" README.md'
    log.debug(command)
    os.system(command)

    command = 'git push'
    log.debug(command)
    os.system(command)

    command = 'git flow init -d'
    log.debug(command)
    os.system(command)

    log.debug('creating Visual Studio Code configuration')
    os.mkdir('.vscode')
    with open(os.path.join('.vscode', 'extensions.json'), 'w') as file:
        file.write(resource_string('wapttools.data', 'vscode_extensions.json'))
    with open(os.path.join('.vscode', 'launch.json'), 'w') as file:
        file.write(resource_string('wapttools.data', 'vscode_launch.json'))
    with open(os.path.join('.vscode', 'settings.json'), 'w') as file:
        file.write(resource_string('wapttools.data', 'vscode_settings.json'))

    log.debug('creating WAPT')
    os.mkdir('WAPT')
    with open(os.path.join('WAPT', 'control'), 'w') as file:
        file.write(resource_string('wapttools.data', 'wapt_control.txt').format(url=gl.url, package=package))
    with open(os.path.join('WAPT', 'icon.png'), 'wb') as file:
        file.write(resource_string('wapttools.data', 'default_icon.png'))

    log.debug('creating config')
    os.mkdir('config')
    open(os.path.join('config', '.gitkeep'), 'a').close()

    log.debug('creating sources')
    os.mkdir('sources')
    with open(os.path.join('sources', '.gitignore'), 'w') as file:
        file.write('*\n')
        file.write('!.gitignore\n')

    log.debug('creating miscellanous files')
    with open('.editorconfig', 'w') as file:
        file.write(resource_string('wapttools.data', 'editorconfig.ini'))

    with open('.env', 'w') as file:
        file.write('PYTHONHOME=C:\\Program Files (x86)\\wapt\n')
        file.write('PYTHONPATH=C:\\Program Files (x86)\\wapt\n')

    with open('.gitignore', 'w') as file:
        file.write('# Ignore genrated files\n')
        file.write('*.pyc\n')
        file.write('WAPT/certificate.crt\n')
        file.write('WAPT/*.sha256\n')
        file.write('WAPT/wapt.psproj\n')

    with open('.gitlab-ci.yml', 'w') as file:
        file.write('include:\n')
        file.write('  - project: "wapt/ci"\n')
        file.write('    file: "/wapt.yml"\n')

    with open('version-check.json', 'w') as file:
        file.write(resource_string('wapttools.data', 'version-check.json'))

    script = resource_string('wapttools.data', 'setup.tmpl')
    with open('setup.py', 'w') as file:
        file.write(script)

    command = 'git add .'
    log.debug(command)
    os.system(command)

    command = 'git commit -m "Initial commit" -a'
    log.debug(command)
    os.system(command)

    command = 'git push --all'
    log.debug(command)
    os.system(command)

    # Create schedule named AutoVersionChecker
    scheds = project.pipelineschedules.list()
    found = False
    for sched in scheds:
        if sched.description == 'AutoVersionChecker':
            found = True

    if not found:
        sched = project.pipelineschedules.create({
            'ref': 'develop',
            'description': 'AutoVersionChecker',
            'cron_timezone': 'Europe/Paris',
            'cron': '{} 5 * * *'.format(random.randint(0, 59)),
            'active': True
        })



import logging
import os

log = logging.getLogger()


def isPackage():
    """ Check if current folder is a WAPT package folder
    """
    if not os.path.isdir('WAPT'):
        log.debug('WAPT folder not found')
        return False

    path = os.path.join('WAPT', 'control')
    if not os.path.isfile(path):
        log.debug('WAPT/control file not found')
        return False

    path = os.path.join('WAPT', 'icon.png')
    if not os.path.isfile(path):
        log.debug('WAPT/icon.png file not found')
        return False

    path = os.path.join('version-check.json')
    if not os.path.isfile(path):
        log.debug('version-check.json file not found')
        return False

    path = os.path.join('setup.py')
    if not os.path.isfile(path):
        log.debug('setup.py file not found')
        return False

    return True



import hashlib
import logging
import os
import sys

log = logging.getLogger()


def sha256sum(filename):
    if not os.path.isfile(filename):
        log.critical('file "{file}" does not exists'.format(file=filename))
        sys.exit(1)

    h = hashlib.sha256()
    b = bytearray(128*1024)
    mv = memoryview(b)
    with open(filename, 'rb', buffering=0) as f:
        for n in iter(lambda: f.readinto(mv), 0):
            h.update(mv[:n])
    computed = h.hexdigest()

    return computed



import logging
import json
import os
import sys

log = logging.getLogger(__name__)


def loadVersionCheck():
    """ Load version-check.json configuration file

    Returns
    -------
    config: dict
        configuration dictionnary
    """

    if not os.path.isfile('version-check.json'):
        log.critical('version-check.json file not found')
        sys.exit(1)

    with open('version-check.json') as file:
        config = json.load(file)

    return config


def loadControl():
    """ Load WAPT/control configuration file

    This only extract name and version.

    Returns
    -------
    config: dict
        configuration dictionnary
    """
    if not os.path.isdir('WAPT'):
        log.critical('WAPT folder not found')
        sys.exit(1)

    path = os.path.join('WAPT', 'control')
    if not os.path.isfile(path):
        log.critical('WAPT/control file not found')
        sys.exit(1)

    config = dict()
    with open(path, 'r') as control:
        for line in control:
            if line.startswith('package'):
                config['package'] = line.split(':')[1].strip()
            if line.startswith('name'):
                config['name'] = line.split(':')[1].strip()
            if line.startswith('version'):
                config['version'] = line.split(':')[1].split('-')[0].strip()
            if line.startswith('architecture'):
                config['architecture'] = line.split(':')[1].strip()
            if line.startswith('maturity'):
                config['maturity'] = line.split(':')[1].strip()
            if line.startswith('homepage'):
                config['homepage'] = line.split(':')[1].strip()

    return config



import fnmatch
import json
import logging
import re
import os
from .control import controlCheck

log = logging.getLogger()


def validate():
    """ Validate a package
    """
    # Validate *.json files
    for root, _, files in os.walk('.'):
        for file in fnmatch.filter(files, '*.json'):
            path = os.path.join(root, file)
            log.debug('Validating json file {}'.format(path))
            with open(path, 'r') as json_file:
                try:
                    json.load(json_file)
                except ValueError:
                    log.error('{} is not a valid json file'.format(path))
                    return False

    # Validate WAPT/control file
    if not controlCheck():
        return False

    log.debug('Check for WAPT/icon.png')
    if not os.path.exists(os.path.join('WAPT', 'icon.png')):
        log.error('WAPT/icon.png missing')
        return False

    # Validate setup.py
    patterns = {
        r'^def install\(': False,
        r'^def uninstall\(': False,
        r'^def session_setup\(': False,
        r'^def audit\(': False,
        r'^def update_package\(': False,
        r'^def download_sources\(': False,
        r'.*__main__': False,
        r'.*wapttools\.commands\(': False,
    }
    with open('setup.py', 'r') as file:
        for line in file:
            if line[-2:] == '\r\n':
                log.error('Found CRLF in setup.py')
                return False

            log.debug('line: {}'.format(line[:-1]))

            for pattern in patterns.keys():
                if re.match(pattern, line):
                    log.error('pattern {} found'.format(pattern))
                    patterns[pattern] = True

    check = True
    for key in patterns.keys():
        if not patterns[key]:
            log.error('{} not found in setup.py'.format(key))
        check &= patterns[key]

    return check
